/*
 * Circular restricted three-body problem
 * 
*/

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <stdint.h>
#include <inttypes.h>
#include <math.h>
#include <omp.h>

#include "r3b.h"



#define SUCCESS 1
#define FAILURE 0



#define N_STEPS ((int)(2.0*M_PI*N_ORB/DT))

#define R1 (sqrt((X0+MU)*(X0+MU)+Y0*Y0))
#define R2 (sqrt((X0-1.0+MU)*(X0-1.0+MU)+Y0*Y0))
#define CJ (X0*X0+Y0*Y0+2.0*(1-MU)/R1+2.0*MU/R2-VX0*VX0-VY0*VY0)


// Phase space coordinates
typedef struct _ph_sp{
	double_t x;
	double_t y;
	double_t vx;
	double_t vy;
} ph_sp;


ph_sp dynamics(ph_sp ph){
	ph_sp outph;
	double_t r1c = ph.x+MU;
	double_t r2c = ph.x-1.0+MU;
	r1c = sqrt(r1c*r1c+ph.y*ph.y);
	r1c *= r1c*r1c;
	r2c = sqrt(r2c*r2c+ph.y*ph.y);
	r2c *= r2c*r2c;
	/* velocity */
	outph.x = ph.vx;
	outph.y = ph.vy;
	/* Force */
	outph.vx = -(1.0-MU)*(ph.x+MU)/r1c-MU*(ph.x-1.0+MU)/r2c;
	outph.vy = -(1.0-MU)*ph.y/r1c-MU*ph.y/r2c;
	/* Efective force (Coriolis correction) */
	outph.vx += 2.0*ph.vy + ph.x;
	outph.vy += -2.0*ph.vx + ph.y;
	return outph;
}


ph_sp sum_ph(ph_sp ph1, ph_sp ph2, double_t a){
	ph_sp outph;
	outph.x = ph1.x+a*ph2.x;
	outph.y = ph1.y+a*ph2.y;
	outph.vx = ph1.vx+a*ph2.vx;
	outph.vy = ph1.vy+a*ph2.vy;
	return outph;
}


// Initial values
ph_sp inic_rk4(void){
	ph_sp temp;
	temp.x = X0;
	temp.y = Y0;
	temp.vx = VX0;
	temp.vy = VY0;
	return temp;
}


// Just for debugging
void princ(ph_sp z){
	fprintf(stderr,"%.4f %.4f %.4f %.4f\n", z.x, z.y, z.vx, z.vy);
}


// rk4
void runge_kutta4(ph_sp *coord_rk4){
	int i;
	ph_sp k1,k2,k3,k4;
	ph_sp temp;

	coord_rk4[0] = inic_rk4();


	for(i=0; i<N_STEPS; i++){
		coord_rk4[i+1] = coord_rk4[i];
		// K1
		k1 = dynamics( coord_rk4[i] );
		coord_rk4[i+1] = sum_ph( coord_rk4[i+1], k1, DT/6.0);
		
		// K2
		k2 = dynamics( sum_ph( coord_rk4[i], k1, DT*0.5) );
		coord_rk4[i+1] = sum_ph( coord_rk4[i+1], k2, DT*2.0/6.0);
		
		// K3
		k3 = dynamics( sum_ph( coord_rk4[i], k2, DT*0.5) );
		coord_rk4[i+1] = sum_ph( coord_rk4[i+1], k3, DT*2.0/6.0);
		
		// K4
		k4 = dynamics( sum_ph( coord_rk4[i], k3, DT) );
		coord_rk4[i+1] = sum_ph( coord_rk4[i+1], k4, DT/6.0);
	}
}


int main(void){
	int i;
	FILE *fp;

	ph_sp *coord_rk4 = (ph_sp*)calloc( N_STEPS+1,  sizeof(ph_sp));
	
	runge_kutta4( coord_rk4 );
	
	fp = fopen("r3b.dat", "w");
	if (NULL == fp) {
		fprintf(stderr, "File failed\n");
		return FAILURE;
	}	
	fprintf(fp,"%.15e\n%.15e\n%d\n", MU, CJ, N_STEPS+1);
	for(i=0; i<N_STEPS+1; i++){
		fprintf(fp,"%e, %e\n", (double)coord_rk4[i].x, (double)coord_rk4[i].y);
	}
	fclose(fp);
	
	free(coord_rk4);
	
	return SUCCESS;
}
